from datetime import datetime
import json

from zabbixproto import PROXY_VERSION, ResponseException, Client


class Proxy:
    def __init__(self, proxy_name, server_ip, server_port):
        self.proxy_name = proxy_name
        self.server_ip = server_ip
        self.server_port = server_port
        self.client = Client(self.server_ip, self.server_port)

    def send_heartbeat(self):
        packet = ProxyHeartbeatPacket(self.proxy_name)

        self.sendWithResponse(packet)

    def sendWithResponse(self, packet):
        resp = self.client.send(packet)
        if 'response' not in resp.data:
            raise ResponseException('no response')
        elif resp.data['response'] != 'success':
            if 'info' in resp.data:
                raise ResponseException("[{}]".format(resp.data['info']))
            else:
                raise ResponseException('unknown failure: {}'.format(resp))
        return (resp.data['info'] if 'info' in resp.data else '')

    def get_config(self):
        packet = ProxyConfigPacket(self.proxy_name)

        resp = self.client.send(packet)

        if 'response' in resp.data and resp.data['response'] == 'failed':
            raise ResponseException(resp.data['info'])

        return resp


class ProxyHeartbeatPacket:
    def __init__(self, proxy_name):
        self.proxy_name = proxy_name
        self.clean()

    def __str__(self):
        return json.dumps(self.packet)

    def clean(self):
        self.packet = {'request': 'proxy heartbeat',
                       'host': self.proxy_name,
                       'version': PROXY_VERSION}


class ProxyConfigPacket:
    def __init__(self, proxy_name):
        self.proxy_name = proxy_name
        self.clean()

    def __str__(self):
        return json.dumps(self.packet)

    def clean(self):
        self.packet = {'request': 'proxy config',
                       'host': self.proxy_name,
                       'version': PROXY_VERSION}


class ProxyDataPacket:
    # This is for version 3.4.X only
    def __init__(self, proxy_name):
        self.proxy_name = proxy_name
        self.clean()

    def __str__(self):
        return json.dumps(self.packet)

    def add_auto_registration(self, host, ip=None, dns=None, port=None, host_metadata=None, clock=datetime.now().timestamp()):
        if (isinstance(clock, int)) or (isinstance(clock, float)):
            metric = {'host': str(host),
                      'clock': int(clock)}

            if ip is not None:
                metric['ip'] = str(ip)
            if dns is not None:
                metric['dns'] = str(dns)
            if port is not None:
                metric['port'] = str(port)
            if host_metadata is not None:
                metric['host_metadata'] = str(host_metadata)
        else:
            raise TypeError('Clock must be unixtime')

        self.packet['auto registration'].append(metric)

    def clean(self):
        self.packet = {'request': 'proxy data',
                       'host': self.proxy_name,
                       'host availability': [],
                       'history data': [],
                       'discovery data': [],
                       'auto registration': [],
                       'tasks': [],
                       'version': PROXY_VERSION}


class ProxyAutoregistrationPacket:
    # This is for version 3.2.X only

    def __init__(self, proxy_name):
        self.proxy_name = proxy_name
        self.clean()

    def __str__(self):
        return json.dumps(self.packet)

    def add(self, host, ip=None, dns=None, port=None, host_metadata=None, clock=datetime.now().timestamp()):
        if (isinstance(clock, int)) or (isinstance(clock, float)):
            metric = {'host': str(host),
                      'clock': int(clock)}

            if ip is not None:
                metric['ip'] = str(ip)
            if dns is not None:
                metric['dns'] = str(dns)
            if port is not None:
                metric['port'] = str(port)
            if host_metadata is not None:
                metric['host_metadata'] = str(host_metadata)
        else:
            raise TypeError('Clock must be unixtime')

        self.packet['data'].append(metric)

    def clean(self):
        self.packet = {'request': 'auto registration',
                       'host': self.proxy_name,
                       'data': [],
                       'clock': datetime.now().timestamp()}


class ProxyHostavailabilityPacket:
    # This is for version 3.2.X only

    def __init__(self, proxy_name):
        self.proxy_name = proxy_name
        self.clean()

    def __str__(self):
        return json.dumps(self.packet)

    def add(self, hostid, available, error='', snmp_available=None, snmp_error='', ipmi_available=None, ipmi_error='', jmx_available=None, jmx_error=''):
        # 	available	number	Zabbix agent availability

        # 0, HOST_AVAILABLE_UNKNOWN - unknown
        # 1, HOST_AVAILABLE_TRUE - available
        # 2, HOST_AVAILABLE_FALSE - unavailable
        # error	string	Zabbix agent error message or empty string
        # snmp_available	number	SNMP agent availability

        # 0, HOST_AVAILABLE_UNKNOWN - unknown
        # 1, HOST_AVAILABLE_TRUE - available
        # 2, HOST_AVAILABLE_FALSE - unavailable
        # snmp_error	string	SNMP agent error message or empty string
        # ipmi_available	number	IPMI agent availability

        # 0, HOST_AVAILABLE_UNKNOWN - unknown
        # 1, HOST_AVAILABLE_TRUE - available
        # 2, HOST_AVAILABLE_FALSE - unavailable
        # ipmi_error	string	IPMI agent error message or empty string
        # jmx_available	number	JMX agent availability

        # 0, HOST_AVAILABLE_UNKNOWN - unknown
        # 1, HOST_AVAILABLE_TRUE - available
        # 2, HOST_AVAILABLE_FALSE - unavailable
        # jmx_error	string	JMX agent error message or empty string

        if not (isinstance(available, int)) or available not in [0, 1, 2]:
            raise TypeError('available must be int 0,1,2')
        if snmp_available is not None and (not (isinstance(snmp_available, int)) or snmp_available not in [0, 1, 2]):
            raise TypeError('snmp_available must be int 0,1,2')
        if ipmi_available is not None and (not (isinstance(ipmi_available, int)) or ipmi_available not in [0, 1, 2]):
            raise TypeError('ipmi_available must be int 0,1,2')
        if jmx_available is not None and (not (isinstance(available, int)) or jmx_available not in [0, 1, 2]):
            raise TypeError('jmx_available must be int 0,1,2')

        metric = {'hostid': int(hostid),
                  'available': int(available)}

        if snmp_available is not None:
            metric['snmp_available'] = int(snmp_available)
            if snmp_available == 2:
                metric['snmp_error'] = str(snmp_error)

        if ipmi_available is not None:
            metric['ipmi_available'] = int(ipmi_available)
            if ipmi_available == 2:
                metric['ipmi_error'] = str(ipmi_error)

        if jmx_available is not None:
            metric['jmx_available'] = int(jmx_available)
            if jmx_available == 2:
                metric['jmx_error'] = str(jmx_error)

        self.packet['data'].append(metric)

    def clean(self):
        self.packet = {'request': 'host availability',
                       'host': self.proxy_name,
                       'data': []}


class ProxyHistorydataPacket:
    # This is for version 3.2.X only
    def __init__(self, proxy_name):
        self.proxy_name = proxy_name
        self.clean()

    def __str__(self):
        return json.dumps(self.packet)

    def add(self, host, key, value, state=0, clock=datetime.now().timestamp()):
        # state
        # 0, ITEM_STATE_NORMAL
        # 1, ITEM_STATE_NOTSUPPORTED
        if not (isinstance(clock, int)) and not (isinstance(clock, float)):
            raise TypeError('Clock must be unixtime')

        if not (isinstance(state, int)) and state not in [0, 1]:
            raise TypeError('state must be int 0,1')

        metric = {'host': str(host),
                  'key': str(key),
                  'state': int(state),
                  'value': str(value),
                  'clock': int(clock)}

        self.packet['data'].append(metric)

    def clean(self):
        self.packet = {'request': 'history data',
                       'host': self.proxy_name,
                       'data': [],
                       'clock': datetime.now().timestamp()}



from .config import PROXY_VERSION, ResponseException
from .client import Client
from .proxypackets import Proxy, ProxyHeartbeatPacket, ProxyConfigPacket, ProxyDataPacket
from .proxypackets import ProxyAutoregistrationPacket, ProxyHostavailabilityPacket, ProxyHistorydataPacket
from .senderpackets import Sender, SenderDataPacket



import json
import socket
import time
import struct


class Response:
    def __init__(self, json):
        self.json = json
        self.data = {}
        self.__parse()

    def __str__(self):
        return self.json

    def __getitem__(self, item):
        return self.data[item]

    def __parse(self):
        try:
            self.data = json.loads(self.json)
        except:
            self.data = {}
            pass


class Client:
    def __init__(self, server='127.0.0.1', port='10051'):
        self.server = server
        self.port = port

    def __str__(self):
        return json.dumps({'server': self.server,
                           'port': self.port},
                          indent=4)

    def send(self, data):
        packetData = str(data).encode('utf-8')
        socket.setdefaulttimeout(60)
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        s.connect((self.server, int(self.port)))

        data_length = len(packetData)
        data_header = struct.pack('<Q', data_length)
        packet = bytes('ZBXD\1', 'utf-8') + data_header + packetData

        s.sendall(packet)
        time.sleep(0.5)

        datalen = None
        body = b''
        idx = 0
        while True:
            idx += 1
            if datalen is not None and len(body) >= datalen:
                break

            data = s.recv(1024)

            if len(data) == 0 and idx >= 5:
                break

            if datalen is None and data[0:5] == b'ZBXD\1':
                datalen = struct.unpack('<Q', data[5:13])[0]
                body = data[13:]
            else:
                body += data

        resp = Response(body.decode('utf-8'))
        s.close()
        return resp



from distutils.core import setup

setup(name='ZabbixProto',
      version='0.0.9',
      description='Zabbix Communication Protocols',
      author='Alen Komic',
      author_email='akomic@gmail.com',
      url='https://github.com/akomic/python-zabbix-proto',
      download_url='https://github.com/akomic/python-zabbix-proto/archive/v0.0.9.tar.gz',
      packages=['zabbixproto'],
      install_requires=[
          'datetime'
      ],
      keywords='Zabbix Protocols Sender Proxy Agent',
      license='Apache Software License',
      classifiers=[
          'Development Status :: 3 - Alpha',
          'License :: OSI Approved :: Apache Software License',
          'Programming Language :: Python :: 3',
          'Topic :: System :: Monitoring',
          'Topic :: System :: Networking :: Monitoring',
          'Topic :: System :: Systems Administration'
      ]
      )



from datetime import datetime
import json

from zabbixproto import PROXY_VERSION, ResponseException, Client


class Sender:
    def __init__(self, server_ip, server_port):
        self.server_ip = server_ip
        self.server_port = server_port
        self.client = Client(self.server_ip, self.server_port)

    def sendWithResponse(self, packet):
        resp = self.client.send(packet)
        if 'response' not in resp.data:
            raise ResponseException('no response')
        elif resp.data['response'] != 'success':
            if 'info' in resp.data:
                raise ResponseException("[{}]".format(resp.data['info']))
            else:
                raise ResponseException('unknown failure: {}'.format(resp))
        return (resp.data['info'] if 'info' in resp.data else '')


class SenderDataPacket:
    def __init__(self, clock=None):
        self.packet = {
            'request': 'sender data',
            'data': [],
            'clock': datetime.now().timestamp() if clock == None else clock
        }

    def __str__(self):
        return json.dumps(self.packet)

    def add(self, host, key, value):
        self.packet['data'].append({
            "host": host,
            "key": key,
            "value": value
        })
